"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

require("core-js/modules/es.array.iterator");

require("core-js/modules/web.dom-collections.iterator");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _classPrivateFieldSet2 = _interopRequireDefault(require("@babel/runtime/helpers/classPrivateFieldSet"));

var _classPrivateFieldGet2 = _interopRequireDefault(require("@babel/runtime/helpers/classPrivateFieldGet"));

var _modules = require("./modules");

var _Player = new WeakMap();

class AudioTape {
  /* ------------------------------------------------------------------ */

  /* Private fields */

  /* ------------------------------------------------------------------ */

  /* Getters */
  get sampleRate() {
    return (0, _classPrivateFieldGet2.default)(this, _Player).sampleRate;
  }

  get active() {
    return (0, _classPrivateFieldGet2.default)(this, _Player).active;
  }

  get totalSeconds() {
    return (0, _classPrivateFieldGet2.default)(this, _Player).totalSeconds;
  }

  get playhead() {
    return (0, _classPrivateFieldGet2.default)(this, _Player).playhead;
  }

  get lookahead() {
    return (0, _classPrivateFieldGet2.default)(this, _Player).lookahead;
  }

  get latency() {
    return (0, _classPrivateFieldGet2.default)(this, _Player).latency;
  }

  get playbackSpeed() {
    return (0, _classPrivateFieldGet2.default)(this, _Player).playbackSpeed;
  }

  get scrubSpeed() {
    return (0, _classPrivateFieldGet2.default)(this, _Player).scrubSpeed;
  }

  get volume() {
    return (0, _classPrivateFieldGet2.default)(this, _Player).volume;
  }

  /* ------------------------------------------------------------------ */

  /* Private Getters */
  get _gainNode() {
    return (0, _classPrivateFieldGet2.default)(this, _Player)._gainNode;
  }

  /* ------------------------------------------------------------------ */

  /* Constructor */
  constructor() {
    var {
      sampleRate = 48e3,
      chunkLength = .02,
      lookahead = 10,
      latency = .2,
      playbackSpeed = 1,
      scrubSpeed = 8,
      LOW_MEMORY_MODE = false
    } = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

    _Player.set(this, {
      writable: true,
      value: void 0
    });

    // Player class instance
    (0, _classPrivateFieldSet2.default)(this, _Player, new _modules.Player({
      sampleRate,
      chunkLength,
      lookahead,
      latency,
      playbackSpeed,
      scrubSpeed,
      Loader: LOW_MEMORY_MODE ? _modules.LoaderLite : _modules.Loader
    })); // Public engine methods

    this.load = (0, _classPrivateFieldGet2.default)(this, _Player).load;
    this.activate = (0, _classPrivateFieldGet2.default)(this, _Player).activate;
    this.deactivate = (0, _classPrivateFieldGet2.default)(this, _Player).deactivate; // Public transport methods

    this.play = (0, _classPrivateFieldGet2.default)(this, _Player).play;
    this.stop = (0, _classPrivateFieldGet2.default)(this, _Player).stop;
    this.rev = (0, _classPrivateFieldGet2.default)(this, _Player).rev;
    this.ff = (0, _classPrivateFieldGet2.default)(this, _Player).ff;
    this.rew = (0, _classPrivateFieldGet2.default)(this, _Player).rew; // Public configuration methods

    this.setPlaybackSpeed = this._setPlaybackSpeed.bind(this);
    this.setScrubSpeed = this._setScrubSpeed.bind(this);
    this.setVolume = this._setVolume.bind(this);
  }

  /* ------------------------------------------------------------------ */

  /* Public Configuration Methods */
  _setPlaybackSpeed(speed) {
    (0, _classPrivateFieldGet2.default)(this, _Player).playbackSpeed = speed;
  }

  _setScrubSpeed(speed) {
    (0, _classPrivateFieldGet2.default)(this, _Player).scrubSpeed = speed;
  }

  _setVolume(val) {
    (0, _classPrivateFieldGet2.default)(this, _Player).volume = val;
  }

}

exports.default = AudioTape;
;