"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

require("core-js/modules/es.array.iterator");

require("core-js/modules/es.array.reverse");

require("core-js/modules/es.array-buffer.slice");

require("core-js/modules/es.promise");

require("core-js/modules/es.typed-array.float32-array");

require("core-js/modules/es.typed-array.to-locale-string");

require("core-js/modules/web.dom-collections.iterator");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _asyncToGenerator2 = _interopRequireDefault(require("@babel/runtime/helpers/asyncToGenerator"));

class Loader {
  /* ------------------------------------------------------------------ */

  /* Constructor */
  constructor() {
    var sampleRate = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 48e3;
    // AudioContext
    this._ctx = new OfflineAudioContext(2, sampleRate, sampleRate); // Audio data

    this._chunkData = []; // Public methods

    this.load = this._load.bind(this);
    this.getSampleDataByChannel = this._getSampleDataByChannel.bind(this);
    this.getReverseSampleDataByChannel = this._getReverseSampleDataByChannel.bind(this);
  }

  /* ------------------------------------------------------------------ */

  /* Public Sample Data Methods */
  _getSampleDataByChannel() {
    var channel = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 0;
    var startSample = arguments.length > 1 ? arguments[1] : undefined;
    var endSample = arguments.length > 2 ? arguments[2] : undefined;
    var outputArray = new Float32Array(endSample - startSample);

    var startChunkIndex = this._chunkData.findIndex(c => c.endSample >= startSample);

    var startChunk = this._chunkData[startChunkIndex];
    var subArray = startChunk.data[channel].subarray(startSample - startChunk.startSample, endSample - startChunk.startSample);
    outputArray.set(subArray, 0);

    if (subArray.length < outputArray.length) {
      // requested samples span multiple chunks
      var overflowSamples = outputArray.length - subArray.length;
      var nextChunk = this._chunkData[startChunkIndex + 1];
      var nextSubArray = nextChunk.data[channel].subarray(0, overflowSamples);
      outputArray.set(nextSubArray, subArray.length);
    }

    ;
    return outputArray;
  }

  _getReverseSampleDataByChannel() {
    var channel = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 0;
    var startSample = arguments.length > 1 ? arguments[1] : undefined;
    var endSample = arguments.length > 2 ? arguments[2] : undefined;
    return this._getSampleDataByChannel(channel, startSample, endSample).reverse();
  }

  /* ------------------------------------------------------------------ */

  /* Public Load Method + Helpers */
  _load() {
    var _arguments = arguments,
        _this = this;

    return (0, _asyncToGenerator2.default)(function* () {
      var files = _arguments.length > 0 && _arguments[0] !== undefined ? _arguments[0] : [];
      var loadCb = _arguments.length > 1 && _arguments[1] !== undefined ? _arguments[1] : null;

      var progressCb = _this._load_genProgressCb(files.length, loadCb);

      var audioBuffers = yield _this._load_fetchAllBuffers(files, progressCb);

      var samplesTotal = _this._load_populateChunkData(audioBuffers, progressCb);

      return samplesTotal;
    })();
  }

  _load_populateChunkData(audioBuffers, progressCb) {
    var currentSample = 0;

    for (var i in audioBuffers) {
      var currentBuffer = audioBuffers[i];
      this._chunkData[i] = {
        length: currentBuffer.length,
        data: [currentBuffer.getChannelData(0), currentBuffer.getChannelData(1)],
        startSample: currentSample
      };
      currentSample += currentBuffer.length;
      this._chunkData[i].endSample = currentSample;
      progressCb();
    }

    ;
    return currentSample;
  }

  _load_fetchAllBuffers() {
    var _arguments2 = arguments,
        _this2 = this;

    return (0, _asyncToGenerator2.default)(function* () {
      var urls = _arguments2.length > 0 && _arguments2[0] !== undefined ? _arguments2[0] : [];
      var progressCb = _arguments2.length > 1 ? _arguments2[1] : undefined;
      var buffers = yield Promise.all(urls.map(url => _this2._load_fetchBuffer(url, progressCb)));
      return buffers.filter(b => b !== null);
    })();
  }

  _load_fetchBuffer() {
    var _arguments3 = arguments,
        _this3 = this;

    return (0, _asyncToGenerator2.default)(function* () {
      var url = _arguments3.length > 0 && _arguments3[0] !== undefined ? _arguments3[0] : '';
      var progressCb = _arguments3.length > 1 ? _arguments3[1] : undefined;

      try {
        var res = yield fetch(url);
        var arrayBuffer = yield res.arrayBuffer();
        var audioBuffer = yield _this3._ctx.decodeAudioData(arrayBuffer);
        progressCb();
        return audioBuffer;
      } catch (err) {
        console.error('_load_fetchBuffer error', err);
        return null;
      }

      ;
    })();
  }

  _load_genProgressCb() {
    var totalFiles = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 0;
    var cb = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : null;
    var loadProgressTotal = totalFiles * 2;
    var loadProgress = 0;
    var progressTickCb = typeof cb === 'function' ? cb : null;
    return () => {
      loadProgress++;

      if (progressTickCb) {
        progressTickCb(loadProgress / loadProgressTotal);
      }

      ;
    };
  }

}

exports.default = Loader;
;