"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

require("core-js/modules/es.array.iterator");

require("core-js/modules/es.array.reverse");

require("core-js/modules/es.array-buffer.slice");

require("core-js/modules/es.promise");

require("core-js/modules/es.typed-array.float32-array");

require("core-js/modules/es.typed-array.to-locale-string");

require("core-js/modules/web.dom-collections.iterator");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _asyncToGenerator2 = _interopRequireDefault(require("@babel/runtime/helpers/asyncToGenerator"));

var _defineProperty2 = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));

class LoaderLite {
  /* ------------------------------------------------------------------ */

  /* Static Properties */
  // chunk cache parameters

  /* ------------------------------------------------------------------ */

  /* Constructor */
  constructor() {
    var sampleRate = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 48e3;
    // AudioContext
    this._ctx = new OfflineAudioContext(2, sampleRate, sampleRate); // Audio data

    this._resBlobs = [];
    this._chunkData = [];
    this._chunkDataBusy = false; // Public methods

    this.load = this._load.bind(this);
    this.getSampleDataByChannel = this._getSampleDataByChannel.bind(this);
    this.getReverseSampleDataByChannel = this._getReverseSampleDataByChannel.bind(this);
  }

  /* ------------------------------------------------------------------ */

  /* Public Sample Data Methods */
  _getSampleDataByChannel() {
    var _arguments = arguments,
        _this = this;

    return (0, _asyncToGenerator2.default)(function* () {
      var channel = _arguments.length > 0 && _arguments[0] !== undefined ? _arguments[0] : 0;
      var startSample = _arguments.length > 1 ? _arguments[1] : undefined;
      var endSample = _arguments.length > 2 ? _arguments[2] : undefined;
      var outputArray = new Float32Array(endSample - startSample);

      var startChunkIndex = _this._chunkData.findIndex(c => c.endSample >= startSample);

      var startChunk = _this._chunkData[startChunkIndex];

      if (startChunk.data === null) {
        yield _this._read_refreshChunkAudioData(startChunkIndex);
      }

      ;
      var subArray = startChunk.data[channel].subarray(startSample - startChunk.startSample, endSample - startChunk.startSample);
      outputArray.set(subArray, 0);

      if (subArray.length < outputArray.length) {
        // requested samples span multiple chunks
        var overflowSamples = outputArray.length - subArray.length;
        var nextChunkIndex = startChunkIndex + 1;
        var nextChunk = _this._chunkData[nextChunkIndex];

        if (nextChunk.data === null) {
          console.warn('Sample buffer underrun!');
          yield _this._read_refreshChunkAudioData(nextChunkIndex);
        }

        ;
        var nextSubArray = nextChunk.data[channel].subarray(0, overflowSamples);
        outputArray.set(nextSubArray, subArray.length);

        _this._read_refreshChunkAudioData(nextChunkIndex);
      }

      ;
      return outputArray;
    })();
  }

  _getReverseSampleDataByChannel() {
    var _arguments2 = arguments,
        _this2 = this;

    return (0, _asyncToGenerator2.default)(function* () {
      var channel = _arguments2.length > 0 && _arguments2[0] !== undefined ? _arguments2[0] : 0;
      var startSample = _arguments2.length > 1 ? _arguments2[1] : undefined;
      var endSample = _arguments2.length > 2 ? _arguments2[2] : undefined;
      return yield _this2.getSampleDataByChannel(channel, startSample, endSample).then(data => data.reverse());
    })();
  }

  /* ------------------------------------------------------------------ */

  /* Read Method + Helpers */
  _read_refreshChunkAudioData(chunkIndex) {
    var _this3 = this;

    return (0, _asyncToGenerator2.default)(function* () {
      if (_this3._chunkDataBusy === chunkIndex) return null;
      yield _this3._read_awaitBusy();
      _this3._chunkDataBusy = chunkIndex;
      var minIndex = chunkIndex - _this3.constructor.ADJACENT_CHUNK_CACHE_LENGTH;
      var maxIndex = chunkIndex + _this3.constructor.ADJACENT_CHUNK_CACHE_LENGTH;

      for (var i = 0; i < _this3._chunkData.length; i++) {
        if (i >= minIndex && i <= maxIndex) {
          if (_this3._chunkData[i].data === null) {
            _this3._chunkData[i].data = yield _this3._read_decodeChunkAudioData(i);
          }

          ;
          continue;
        }

        ;

        if (_this3._chunkData[i].data !== null) {
          _this3._chunkData[i].data = null;
        }

        ;
      }

      ;
      _this3._chunkDataBusy = false;
    })();
  }

  _read_decodeChunkAudioData(chunkIndex) {
    var _this4 = this;

    return (0, _asyncToGenerator2.default)(function* () {
      var tempResBuffer = yield _this4._resBlobs[chunkIndex].arrayBuffer();
      var tempAudioBuffer = yield _this4._ctx.decodeAudioData(tempResBuffer);
      return [tempAudioBuffer.getChannelData(0), tempAudioBuffer.getChannelData(1)];
    })();
  }

  _read_awaitBusy() {
    var _this5 = this;

    return (0, _asyncToGenerator2.default)(function* () {
      while (_this5._chunkDataBusy !== false) {
        yield new Promise(res => setTimeout(res, 16));
      }

      ;
      return true;
    })();
  }

  /* ------------------------------------------------------------------ */

  /* Public Load Method + Helpers */
  _load() {
    var _arguments3 = arguments,
        _this6 = this;

    return (0, _asyncToGenerator2.default)(function* () {
      var files = _arguments3.length > 0 && _arguments3[0] !== undefined ? _arguments3[0] : [];
      var loadCb = _arguments3.length > 1 && _arguments3[1] !== undefined ? _arguments3[1] : null;
      var params = _arguments3.length > 2 && _arguments3[2] !== undefined ? _arguments3[2] : {};
      var {
        exact = false
      } = params;

      var progressCb = _this6._load_genProgressCb(files.length, loadCb);

      _this6._resBlobs = yield _this6._load_fetchAllBlobs(files, progressCb);
      var samplesTotal = yield _this6._load_populateChunkData(progressCb, exact);
      return samplesTotal;
    })();
  }

  _load_populateChunkData(progressCb) {
    var _arguments4 = arguments,
        _this7 = this;

    return (0, _asyncToGenerator2.default)(function* () {
      var exact = _arguments4.length > 1 && _arguments4[1] !== undefined ? _arguments4[1] : false;
      var currentSample = 0,
          tempResBuffer,
          tempAudioBuffer;

      for (var i in _this7._resBlobs) {
        _this7._chunkData[i] = {};
        _this7._chunkData[i].startSample = currentSample;

        if (i < 2 || !exact) {
          tempResBuffer = yield _this7._resBlobs[i].arrayBuffer();
          tempAudioBuffer = yield _this7._ctx.decodeAudioData(tempResBuffer);
        }

        ;

        if (i < 2) {
          _this7._chunkData[i].data = [tempAudioBuffer.getChannelData(0), tempAudioBuffer.getChannelData(1)];
        } else {
          _this7._chunkData[i].data = null;
        }

        ;
        currentSample += tempAudioBuffer.length;
        _this7._chunkData[i].endSample = currentSample - 1;
        progressCb();
      }

      ;
      return currentSample;
    })();
  }

  _load_fetchAllBlobs() {
    var _arguments5 = arguments,
        _this8 = this;

    return (0, _asyncToGenerator2.default)(function* () {
      var urls = _arguments5.length > 0 && _arguments5[0] !== undefined ? _arguments5[0] : [];
      var progressCb = _arguments5.length > 1 ? _arguments5[1] : undefined;
      var blobs = yield Promise.all(urls.map(url => _this8._load_fetchBlob(url, progressCb)));
      return blobs.filter(b => b !== null);
    })();
  }

  _load_fetchBlob() {
    var _arguments6 = arguments;
    return (0, _asyncToGenerator2.default)(function* () {
      var url = _arguments6.length > 0 && _arguments6[0] !== undefined ? _arguments6[0] : '';
      var progressCb = _arguments6.length > 1 ? _arguments6[1] : undefined;

      try {
        var res = yield fetch(url);
        var blob = yield res.blob();
        progressCb();
        return blob;
      } catch (err) {
        console.error('_load_fetchBlob error', err);
        return null;
      }

      ;
    })();
  }

  _load_genProgressCb() {
    var totalFiles = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 0;
    var cb = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : null;
    var loadProgressTotal = totalFiles * 2;
    var loadProgress = 0;
    var progressTickCb = typeof cb === 'function' ? cb : null;
    return () => {
      loadProgress++;

      if (progressTickCb) {
        progressTickCb(loadProgress / loadProgressTotal);
      }

      ;
    };
  }

}

exports.default = LoaderLite;
(0, _defineProperty2.default)(LoaderLite, "ADJACENT_CHUNK_CACHE_LENGTH", 2);
;