"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

require("core-js/modules/es.array.iterator");

require("core-js/modules/es.array.splice");

require("core-js/modules/es.promise");

require("core-js/modules/web.dom-collections.iterator");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _asyncToGenerator2 = _interopRequireDefault(require("@babel/runtime/helpers/asyncToGenerator"));

var _defineProperty2 = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));

var _classPrivateFieldSet2 = _interopRequireDefault(require("@babel/runtime/helpers/classPrivateFieldSet"));

var _classPrivateFieldGet2 = _interopRequireDefault(require("@babel/runtime/helpers/classPrivateFieldGet"));

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { (0, _defineProperty2.default)(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

var _ctx = new WeakMap();

var _masterGain = new WeakMap();

var _bufferGain = new WeakMap();

var _sampleRate = new WeakMap();

var _rampChunkSamples = new WeakMap();

var _chunkSamples = new WeakMap();

var _lookaheadSeconds = new WeakMap();

var _scheduledSeconds = new WeakMap();

var _pendingSeconds = new WeakMap();

var _uiLatency = new WeakMap();

var _playbackSpeeds = new WeakMap();

var _state = new WeakMap();

var _scheduledQueue = new WeakMap();

var _pendingQueue = new WeakMap();

var _tickInterval = new WeakMap();

var _tickTimeout = new WeakMap();

var _tickCb = new WeakMap();

var _Loader = new WeakMap();

var _totalSamples = new WeakMap();

var _totalSeconds = new WeakMap();

class Player {
  /* ------------------------------------------------------------------ */

  /* Private fields */

  /* ------------------------------------------------------------------ */

  /* Static Properties */
  // default constructor argument parameter values
  // minimum parameter values
  // chunk sample size constants
  // pre-defined parameter objects for transport state changes

  /* ------------------------------------------------------------------ */

  /* Static Methods */
  static calcRampChunkSamples(sampleRate, targetRampChunkSeconds) {
    var targetRampChunkSamples = Math.floor(sampleRate * targetRampChunkSeconds);
    var targetRampChunkKBins = Math.round(targetRampChunkSamples / this.KBIN_SAMPLES);
    var rampChunkKBins = Math.max(this.MIN_RAMP_CHUNK_KBINS, targetRampChunkKBins);
    var rampChunkSamples = rampChunkKBins * this.KBIN_SAMPLES;
    return rampChunkSamples;
  }

  static clampMinValidNumber(val, minVal) {
    if (typeof val === 'number' && val >= minVal) {
      return val;
    }

    ;
    return minVal;
  }

  static calcRampDuration(startSpeed, endSpeed) {
    return Math.sqrt(Math.abs(startSpeed - endSpeed));
  }

  static parseLoadSrc(src) {
    if (Array.isArray(src)) {
      if (!src.every(url => typeof url === 'string')) {
        throw new TypeError('Source URLs must be strings');
      }

      ;
      return src;
    }

    ;

    if (typeof src === 'string') {
      return [src];
    }

    ;
    return [];
  }

  /* ------------------------------------------------------------------ */

  /* Getters */
  get sampleRate() {
    return (0, _classPrivateFieldGet2.default)(this, _sampleRate);
  }

  get active() {
    return this._ctxActive && this._engineActive;
  }

  get totalSeconds() {
    return (0, _classPrivateFieldGet2.default)(this, _totalSeconds);
  }

  get playhead() {
    if (!(0, _classPrivateFieldGet2.default)(this, _scheduledQueue).length) {
      return (0, _classPrivateFieldGet2.default)(this, _state).resumeSample / (0, _classPrivateFieldGet2.default)(this, _sampleRate);
    }

    ;
    var nowChunk = (0, _classPrivateFieldGet2.default)(this, _scheduledQueue)[0];
    var elapsedSeconds = ((0, _classPrivateFieldGet2.default)(this, _ctx).currentTime - nowChunk.ctxStartTime) * (nowChunk.ctxPlaybackSpeed * nowChunk.direction);
    return nowChunk.srcStartSeconds + elapsedSeconds;
  }

  get lookahead() {
    return (0, _classPrivateFieldGet2.default)(this, _lookaheadSeconds);
  }

  get latency() {
    return (0, _classPrivateFieldGet2.default)(this, _scheduledSeconds);
  }

  get playbackSpeed() {
    return (0, _classPrivateFieldGet2.default)(this, _playbackSpeeds).base;
  }

  get scrubSpeed() {
    return (0, _classPrivateFieldGet2.default)(this, _playbackSpeeds).scrub;
  }

  get volume() {
    return (0, _classPrivateFieldGet2.default)(this, _masterGain).gain.value;
  }

  /* ------------------------------------------------------------------ */

  /* Private Getters */
  get _ctxActive() {
    return (0, _classPrivateFieldGet2.default)(this, _ctx).state === 'running';
  }

  get _engineActive() {
    return !!(0, _classPrivateFieldGet2.default)(this, _tickTimeout);
  }

  get _lastScheduledChunk() {
    return (0, _classPrivateFieldGet2.default)(this, _scheduledQueue)[(0, _classPrivateFieldGet2.default)(this, _scheduledQueue).length - 1];
  }

  get _lastPendingChunk() {
    return (0, _classPrivateFieldGet2.default)(this, _pendingQueue)[(0, _classPrivateFieldGet2.default)(this, _pendingQueue).length - 1];
  }

  get _gainNode() {
    return (0, _classPrivateFieldGet2.default)(this, _masterGain);
  }

  /* ------------------------------------------------------------------ */

  /* Setters */
  set lookahead(seconds) {
    (0, _classPrivateFieldSet2.default)(this, _lookaheadSeconds, this.constructor.clampMinValidNumber(seconds, this.constructor.MIN_LOOKAHEAD_SECONDS));
    (0, _classPrivateFieldSet2.default)(this, _pendingSeconds, (0, _classPrivateFieldGet2.default)(this, _lookaheadSeconds) - (0, _classPrivateFieldGet2.default)(this, _scheduledSeconds));
  }

  set latency(seconds) {
    (0, _classPrivateFieldSet2.default)(this, _scheduledSeconds, this.constructor.clampMinValidNumber(seconds, this.constructor.MIN_SCHEDULED_SECONDS));
    (0, _classPrivateFieldSet2.default)(this, _pendingSeconds, (0, _classPrivateFieldGet2.default)(this, _lookaheadSeconds) - (0, _classPrivateFieldGet2.default)(this, _scheduledSeconds));
    (0, _classPrivateFieldSet2.default)(this, _uiLatency, (0, _classPrivateFieldGet2.default)(this, _scheduledSeconds) / 5);
  }

  set playbackSpeed(speed) {
    (0, _classPrivateFieldGet2.default)(this, _playbackSpeeds).base = this._clampSpeed(speed);

    if (!(0, _classPrivateFieldGet2.default)(this, _state).busy && !(0, _classPrivateFieldGet2.default)(this, _state).scrubbing && (0, _classPrivateFieldGet2.default)(this, _state).playing) {
      if ((0, _classPrivateFieldGet2.default)(this, _state).direction === 1) {
        this._transport_setState(_objectSpread({
          delta: true
        }, this.constructor.TRANSPORT.play));
      }

      ;

      if ((0, _classPrivateFieldGet2.default)(this, _state).direction === -1) {
        this._transport_setState(_objectSpread({
          delta: true
        }, this.constructor.TRANSPORT.rev));
      }

      ;
    }

    ;
  }

  set scrubSpeed(speed) {
    (0, _classPrivateFieldGet2.default)(this, _playbackSpeeds).scrub = this._clampSpeed(speed);

    if (!(0, _classPrivateFieldGet2.default)(this, _state).busy && (0, _classPrivateFieldGet2.default)(this, _state).scrubbing && (0, _classPrivateFieldGet2.default)(this, _state).playing) {
      if ((0, _classPrivateFieldGet2.default)(this, _state).direction === 1) {
        this._transport_setState(_objectSpread({
          delta: true
        }, this.constructor.TRANSPORT.ff));
      }

      ;

      if ((0, _classPrivateFieldGet2.default)(this, _state).direction === -1) {
        this._transport_setState(_objectSpread({
          delta: true
        }, this.constructor.TRANSPORT.rew));
      }

      ;
    }

    ;
  }

  set volume(val) {
    var safeVal = this.constructor.clampMinValidNumber(val, 0);
    var now = (0, _classPrivateFieldGet2.default)(this, _ctx).currentTime;
    var rampEnd = now + (0, _classPrivateFieldGet2.default)(this, _uiLatency);
    (0, _classPrivateFieldGet2.default)(this, _masterGain).gain.cancelScheduledValues(now);
    (0, _classPrivateFieldGet2.default)(this, _masterGain).gain.linearRampToValueAtTime(safeVal, rampEnd);
  }

  /* ------------------------------------------------------------------ */

  /* Constructor */
  constructor() {
    var {
      /* fixed at instantiation */
      sampleRate = this.constructor.DEFAULT_SAMPLE_RATE,
      chunkLength = this.constructor.DEFAULT_CHUNK_LENGTH,

      /* can change via setters */
      lookahead = this.constructor.DEFAULT_LOOKAHEAD,
      latency = this.constructor.DEFAULT_LATENCY,
      playbackSpeed = this.constructor.DEFAULT_PLAYBACK_SPEED,
      scrubSpeed = this.constructor.DEFAULT_SCRUB_SPEED,
      Loader
    } = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

    _ctx.set(this, {
      writable: true,
      value: void 0
    });

    _masterGain.set(this, {
      writable: true,
      value: void 0
    });

    _bufferGain.set(this, {
      writable: true,
      value: void 0
    });

    _sampleRate.set(this, {
      writable: true,
      value: void 0
    });

    _rampChunkSamples.set(this, {
      writable: true,
      value: void 0
    });

    _chunkSamples.set(this, {
      writable: true,
      value: void 0
    });

    _lookaheadSeconds.set(this, {
      writable: true,
      value: void 0
    });

    _scheduledSeconds.set(this, {
      writable: true,
      value: void 0
    });

    _pendingSeconds.set(this, {
      writable: true,
      value: void 0
    });

    _uiLatency.set(this, {
      writable: true,
      value: void 0
    });

    _playbackSpeeds.set(this, {
      writable: true,
      value: void 0
    });

    _state.set(this, {
      writable: true,
      value: void 0
    });

    _scheduledQueue.set(this, {
      writable: true,
      value: void 0
    });

    _pendingQueue.set(this, {
      writable: true,
      value: void 0
    });

    _tickInterval.set(this, {
      writable: true,
      value: void 0
    });

    _tickTimeout.set(this, {
      writable: true,
      value: void 0
    });

    _tickCb.set(this, {
      writable: true,
      value: void 0
    });

    _Loader.set(this, {
      writable: true,
      value: void 0
    });

    _totalSamples.set(this, {
      writable: true,
      value: void 0
    });

    _totalSeconds.set(this, {
      writable: true,
      value: void 0
    });

    (0, _classPrivateFieldSet2.default)(this, _ctx, new AudioContext({
      sampleRate
    })); // GainNode instantiation

    (0, _classPrivateFieldSet2.default)(this, _masterGain, (0, _classPrivateFieldGet2.default)(this, _ctx).createGain());
    (0, _classPrivateFieldGet2.default)(this, _masterGain).connect((0, _classPrivateFieldGet2.default)(this, _ctx).destination);
    (0, _classPrivateFieldGet2.default)(this, _masterGain).gain.value = 1;
    (0, _classPrivateFieldSet2.default)(this, _bufferGain, (0, _classPrivateFieldGet2.default)(this, _ctx).createGain());
    (0, _classPrivateFieldGet2.default)(this, _bufferGain).connect((0, _classPrivateFieldGet2.default)(this, _masterGain));
    (0, _classPrivateFieldGet2.default)(this, _bufferGain).gain.value = 0; // Engine timing constants

    (0, _classPrivateFieldSet2.default)(this, _sampleRate, (0, _classPrivateFieldGet2.default)(this, _ctx).sampleRate);
    (0, _classPrivateFieldSet2.default)(this, _rampChunkSamples, this.constructor.calcRampChunkSamples((0, _classPrivateFieldGet2.default)(this, _sampleRate), chunkLength));
    (0, _classPrivateFieldSet2.default)(this, _chunkSamples, (0, _classPrivateFieldGet2.default)(this, _rampChunkSamples) * 2);
    (0, _classPrivateFieldSet2.default)(this, _lookaheadSeconds, this.constructor.clampMinValidNumber(lookahead, this.constructor.MIN_LOOKAHEAD_SECONDS));
    (0, _classPrivateFieldSet2.default)(this, _scheduledSeconds, this.constructor.clampMinValidNumber(latency, this.constructor.MIN_SCHEDULED_SECONDS));
    (0, _classPrivateFieldSet2.default)(this, _pendingSeconds, (0, _classPrivateFieldGet2.default)(this, _lookaheadSeconds) - (0, _classPrivateFieldGet2.default)(this, _scheduledSeconds));
    (0, _classPrivateFieldSet2.default)(this, _uiLatency, (0, _classPrivateFieldGet2.default)(this, _scheduledSeconds) / 5);
    (0, _classPrivateFieldSet2.default)(this, _playbackSpeeds, {
      base: this._clampSpeed(playbackSpeed),
      min: this._clampSpeed(this.constructor.MIN_PLAYBACK_SPEED),
      scrub: this._clampSpeed(scrubSpeed)
    }); // Engine state

    (0, _classPrivateFieldSet2.default)(this, _state, {
      resumeSample: 0,
      playing: 0,
      scrubbing: 0,
      direction: 1,
      busy: false
    }); // Engine queues

    (0, _classPrivateFieldSet2.default)(this, _scheduledQueue, []);
    (0, _classPrivateFieldSet2.default)(this, _pendingQueue, []); // Engine runtime

    (0, _classPrivateFieldSet2.default)(this, _tickInterval, (0, _classPrivateFieldGet2.default)(this, _rampChunkSamples) / (0, _classPrivateFieldGet2.default)(this, _sampleRate) * 1e3);
    (0, _classPrivateFieldSet2.default)(this, _tickTimeout, null);
    (0, _classPrivateFieldSet2.default)(this, _tickCb, this._tick.bind(this)); // Loader + metadata

    (0, _classPrivateFieldSet2.default)(this, _Loader, new Loader((0, _classPrivateFieldGet2.default)(this, _sampleRate)));
    (0, _classPrivateFieldSet2.default)(this, _totalSamples, 0);
    (0, _classPrivateFieldSet2.default)(this, _totalSeconds, 0); // Public engine methods

    this.load = this._load.bind(this);
    this.activate = this._activate.bind(this);
    this.deactivate = this._deactivate.bind(this); // Public transport methods

    this.play = this._play.bind(this);
    this.stop = this._stop.bind(this);
    this.rev = this._rev.bind(this);
    this.ff = this._ff.bind(this);
    this.rew = this._rew.bind(this);
  }

  /* ------------------------------------------------------------------ */

  /* Public Engine Methods */
  _load(src) {
    var _arguments = arguments,
        _this = this;

    return (0, _asyncToGenerator2.default)(function* () {
      var safeSrc = _this.constructor.parseLoadSrc(src);

      for (var _len = _arguments.length, args = new Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
        args[_key - 1] = _arguments[_key];
      }

      (0, _classPrivateFieldSet2.default)(_this, _totalSamples, yield (0, _classPrivateFieldGet2.default)(_this, _Loader).load(safeSrc, ...args));
      (0, _classPrivateFieldSet2.default)(_this, _totalSeconds, (0, _classPrivateFieldGet2.default)(_this, _totalSamples) / (0, _classPrivateFieldGet2.default)(_this, _sampleRate));
      return true;
    })();
  }

  _activate() {
    if (!this._ctxActive) {
      (0, _classPrivateFieldGet2.default)(this, _ctx).resume();
    }

    ;

    if (!this._engineActive) {
      (0, _classPrivateFieldGet2.default)(this, _tickCb).call(this);
    }

    ;
  }

  _deactivate() {
    if (this._ctxActive) {
      (0, _classPrivateFieldGet2.default)(this, _ctx).suspend();
    }

    ;

    if (this._engineActive) {
      clearTimeout((0, _classPrivateFieldGet2.default)(this, _tickTimeout));
    }

    ;
  }

  /* ------------------------------------------------------------------ */

  /* Public Transport Methods */
  _play() {
    var _this2 = this;

    return (0, _asyncToGenerator2.default)(function* () {
      yield _this2._transport_setState(_this2.constructor.TRANSPORT.play);
      return true;
    })();
  }

  _stop() {
    var _this3 = this;

    return (0, _asyncToGenerator2.default)(function* () {
      yield _this3._transport_setState(_this3.constructor.TRANSPORT.stop);
      return true;
    })();
  }

  _rev() {
    var _this4 = this;

    return (0, _asyncToGenerator2.default)(function* () {
      yield _this4._transport_setState(_this4.constructor.TRANSPORT.rev);
      return true;
    })();
  }

  _ff() {
    var _this5 = this;

    return (0, _asyncToGenerator2.default)(function* () {
      yield _this5._transport_setState(_this5.constructor.TRANSPORT.ff);
      return true;
    })();
  }

  _rew() {
    var _this6 = this;

    return (0, _asyncToGenerator2.default)(function* () {
      yield _this6._transport_setState(_this6.constructor.TRANSPORT.rew);
      return true;
    })();
  }

  /* ------------------------------------------------------------------ */

  /* Engine Tick Callback */
  _tick() {
    var _this7 = this;

    return (0, _asyncToGenerator2.default)(function* () {
      yield _this7._queue_tick();
      (0, _classPrivateFieldSet2.default)(_this7, _tickTimeout, setTimeout((0, _classPrivateFieldGet2.default)(_this7, _tickCb), (0, _classPrivateFieldGet2.default)(_this7, _tickInterval)));
    })();
  }

  /* ------------------------------------------------------------------ */

  /* Queue Scheduling + Helpers  */

  /* queue runtime */
  _queue_tick() {
    var _this8 = this;

    return (0, _asyncToGenerator2.default)(function* () {
      if ((0, _classPrivateFieldGet2.default)(_this8, _state).busy) return null;
      var currentTime = (0, _classPrivateFieldGet2.default)(_this8, _ctx).currentTime;

      _this8._queue_clearScheduledHead(currentTime);

      _this8._queue_schedulePendingChunks(currentTime);

      if ((0, _classPrivateFieldGet2.default)(_this8, _state).playing === 1) {
        yield _this8._queue_refillPendingChunks();
      }

      ;
    })();
  }

  /* remove chunks that have finished playing from top of scheduled queue */
  _queue_clearScheduledHead(ctxTime) {
    if (!(0, _classPrivateFieldGet2.default)(this, _scheduledQueue).length) return null;
    var currentScheduledChunk = (0, _classPrivateFieldGet2.default)(this, _scheduledQueue)[0];
    var nextStartTime = currentScheduledChunk.ctxNextStartTime;

    while (nextStartTime < ctxTime) {
      (0, _classPrivateFieldGet2.default)(this, _state).resumeSample = currentScheduledChunk.srcEndSample;
      (0, _classPrivateFieldGet2.default)(this, _scheduledQueue).shift();
      currentScheduledChunk = (0, _classPrivateFieldGet2.default)(this, _scheduledQueue)[0];
      if (!currentScheduledChunk) break;
      nextStartTime = currentScheduledChunk.ctxNextStartTime;
    }

    ;
  }

  /* schedule chunk for playback and assign values to returned chunk object */
  _queue_scheduleChunk() {
    var chunk = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
    var startTimeSample = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 0;
    var playbackSeconds = chunk.srcLengthSamples / (0, _classPrivateFieldGet2.default)(this, _sampleRate) / chunk.ctxPlaybackSpeed;
    chunk.ctxLengthSamples = Math.floor(playbackSeconds * (0, _classPrivateFieldGet2.default)(this, _sampleRate));
    chunk.ctxPlaybackSpeed = chunk.srcLengthSamples / chunk.ctxLengthSamples;
    chunk.ctxStartTime = startTimeSample / (0, _classPrivateFieldGet2.default)(this, _sampleRate);
    chunk.ctxNextStartTimeSample = Math.floor(startTimeSample + chunk.ctxLengthSamples);
    chunk.ctxNextStartTime = chunk.ctxNextStartTimeSample / (0, _classPrivateFieldGet2.default)(this, _sampleRate);
    chunk.node = (0, _classPrivateFieldGet2.default)(this, _ctx).createBufferSource();
    chunk.node.buffer = chunk.buffer;
    chunk.node.playbackRate.value = chunk.ctxPlaybackSpeed;
    chunk.node.connect((0, _classPrivateFieldGet2.default)(this, _bufferGain));
    chunk.node.start(chunk.ctxStartTime, 0);
    return chunk;
  }

  /* schedule gain change */
  _queue_scheduleGain(ctxEndTime, targetVal) {
    (0, _classPrivateFieldGet2.default)(this, _bufferGain).gain.linearRampToValueAtTime(targetVal, ctxEndTime);
  }

  /* schedule chunks and shift from pending to scheduled */
  _queue_schedulePendingChunks(ctxTime) {
    var _lastScheduledChunk$c, _lastScheduledChunk, _lastScheduledChunk$c2, _lastScheduledChunk2;

    if (!(0, _classPrivateFieldGet2.default)(this, _pendingQueue).length) return null;
    var scheduledChunksFillTarget = (ctxTime + (0, _classPrivateFieldGet2.default)(this, _scheduledSeconds)) * (0, _classPrivateFieldGet2.default)(this, _sampleRate);
    var lastScheduledChunk = this._lastScheduledChunk;
    var lastCtxGain = (_lastScheduledChunk$c = (_lastScheduledChunk = lastScheduledChunk) === null || _lastScheduledChunk === void 0 ? void 0 : _lastScheduledChunk.ctxGain) !== null && _lastScheduledChunk$c !== void 0 ? _lastScheduledChunk$c : 0;
    var nextStartTimeSample = (_lastScheduledChunk$c2 = (_lastScheduledChunk2 = lastScheduledChunk) === null || _lastScheduledChunk2 === void 0 ? void 0 : _lastScheduledChunk2.ctxNextStartTimeSample) !== null && _lastScheduledChunk$c2 !== void 0 ? _lastScheduledChunk$c2 : Math.round(this._clampClock(ctxTime + (0, _classPrivateFieldGet2.default)(this, _uiLatency)) * (0, _classPrivateFieldGet2.default)(this, _sampleRate));

    while (nextStartTimeSample < scheduledChunksFillTarget) {
      var nextChunk = (0, _classPrivateFieldGet2.default)(this, _pendingQueue).shift();
      if (!nextChunk) break;
      lastScheduledChunk = this._queue_scheduleChunk(nextChunk, nextStartTimeSample);
      (0, _classPrivateFieldGet2.default)(this, _scheduledQueue).push(lastScheduledChunk);

      if (lastCtxGain !== lastScheduledChunk.ctxGain) {
        this._queue_scheduleGain(lastScheduledChunk.ctxNextStartTime, lastScheduledChunk.ctxGain);
      }

      ;
      nextStartTimeSample = lastScheduledChunk.ctxNextStartTimeSample;
      lastCtxGain = lastScheduledChunk.ctxGain;
    }

    ;
  }

  /* repopulate pending queue with new chunks */
  _queue_refillPendingChunks() {
    var _this9 = this;

    return (0, _asyncToGenerator2.default)(function* () {
      var _this9$_lastScheduled;

      var lastScheduledEndSample = (_this9$_lastScheduled = _this9._lastScheduledChunk) === null || _this9$_lastScheduled === void 0 ? void 0 : _this9$_lastScheduled.srcEndSample;
      var lastPendingChunk = _this9._lastPendingChunk;
      if (!lastScheduledEndSample || !lastPendingChunk) return null;
      var direction = lastPendingChunk.direction;
      var pendingChunksFillTarget = lastScheduledEndSample + direction * (0, _classPrivateFieldGet2.default)(_this9, _pendingSeconds) * (0, _classPrivateFieldGet2.default)(_this9, _sampleRate);
      var speed = lastPendingChunk.ctxPlaybackSpeed;
      var test = direction < 0 ? () => nextStartSample > pendingChunksFillTarget : () => nextStartSample < pendingChunksFillTarget;
      var nextChunk = lastPendingChunk;
      var nextStartSample = nextChunk.srcEndSample;

      while (test()) {
        var startSample = nextStartSample;
        nextStartSample += (0, _classPrivateFieldGet2.default)(_this9, _chunkSamples) * direction;
        nextChunk = yield _this9._chunk_create(startSample, nextStartSample, speed);
        if (!nextChunk) break;
        (0, _classPrivateFieldGet2.default)(_this9, _pendingQueue).push(nextChunk);
      }

      ;
    })();
  }

  /* depopulate pending queue */
  _queue_clearPending() {
    (0, _classPrivateFieldGet2.default)(this, _pendingQueue).splice(0, (0, _classPrivateFieldGet2.default)(this, _pendingQueue).length);
  }

  /* ------------------------------------------------------------------ */

  /* Chunk Creation + Helpers  */

  /* create new playback chunk object and populate buffer with sample data */
  _chunk_create(startSample, endSample, speed) {
    var _arguments2 = arguments,
        _this10 = this;

    return (0, _asyncToGenerator2.default)(function* () {
      var gain = _arguments2.length > 3 && _arguments2[3] !== undefined ? _arguments2[3] : 1;

      try {
        var clamped = _this10._chunk_validateSampleRange(startSample, endSample);

        if (!clamped) return null;
        var buffer = yield _this10._chunk_getBuffer(clamped.start, clamped.end);
        return {
          node: null,
          buffer: buffer,
          direction: clamped.start < clamped.end ? 1 : -1,
          srcStartSeconds: clamped.start / (0, _classPrivateFieldGet2.default)(_this10, _sampleRate),
          srcStartSample: clamped.start,
          srcEndSample: clamped.end,
          srcLengthSamples: buffer.length,
          ctxPlaybackSpeed: speed,
          ctxStartTime: null,
          ctxNextStartTime: null,
          ctxNextStartTimeSample: null,
          ctxLengthSamples: null,
          ctxGain: gain
        };
      } catch (err) {
        console.error('_chunk_create', err);
        return null;
      }

      ;
    })();
  }

  /* clamp start + end sample values to valid integers and get total sample length */
  _chunk_validateSampleRange(startSample, endSample) {
    var start = this._clampSample(startSample);

    var end = this._clampSample(endSample);

    var srcLengthSamples = Math.abs(end - start);
    if (!srcLengthSamples) return null;
    return {
      start,
      end
    };
  }

  /* create new buffer and populate with sample data from specified range */
  _chunk_getBuffer(startSample, endSample) {
    var _this11 = this;

    return (0, _asyncToGenerator2.default)(function* () {
      var buffer;

      if (endSample < startSample) {
        buffer = (0, _classPrivateFieldGet2.default)(_this11, _ctx).createBuffer(2, startSample - endSample, (0, _classPrivateFieldGet2.default)(_this11, _sampleRate));
        buffer.copyToChannel(yield (0, _classPrivateFieldGet2.default)(_this11, _Loader).getReverseSampleDataByChannel(0, endSample, startSample), 0, 0);
        buffer.copyToChannel(yield (0, _classPrivateFieldGet2.default)(_this11, _Loader).getReverseSampleDataByChannel(1, endSample, startSample), 1, 0);
      } else {
        buffer = (0, _classPrivateFieldGet2.default)(_this11, _ctx).createBuffer(2, endSample - startSample, (0, _classPrivateFieldGet2.default)(_this11, _sampleRate));
        buffer.copyToChannel(yield (0, _classPrivateFieldGet2.default)(_this11, _Loader).getSampleDataByChannel(0, startSample, endSample), 0, 0);
        buffer.copyToChannel(yield (0, _classPrivateFieldGet2.default)(_this11, _Loader).getSampleDataByChannel(1, startSample, endSample), 1, 0);
      }

      ;
      return buffer;
    })();
  }

  /* ------------------------------------------------------------------ */

  /* Transport State Management + Helpers */
  _transport_setState() {
    var _arguments3 = arguments,
        _this12 = this;

    return (0, _asyncToGenerator2.default)(function* () {
      var nextState = _arguments3.length > 0 && _arguments3[0] !== undefined ? _arguments3[0] : {};

      var validNextState = _this12._transport_validateNextState(nextState);

      if (!validNextState) return null;
      yield _this12._transport_awaitBusyState();
      (0, _classPrivateFieldGet2.default)(_this12, _state).busy = true;
      yield _this12._transport_scheduleDeltaChunks(validNextState);

      _this12._transport_applyNextState(validNextState);

      (0, _classPrivateFieldGet2.default)(_this12, _state).busy = false;
    })();
  }

  _transport_awaitBusyState() {
    var _this13 = this;

    return (0, _asyncToGenerator2.default)(function* () {
      while ((0, _classPrivateFieldGet2.default)(_this13, _state).busy === true) {
        yield new Promise(res => setTimeout(res, 16));
      }

      ;
      return true;
    })();
  }

  /* validate nextState object and return null if invalid */
  _transport_validateNextState() {
    var nextState = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
    var nextDirection = nextState === null || nextState === void 0 ? void 0 : nextState.direction;

    switch (true) {
      // no state change, abort
      case Object.entries(nextState).every((_ref) => {
        var [key, val] = _ref;
        return (0, _classPrivateFieldGet2.default)(this, _state)[key] === val;
      }):
        return null;
      // attempting to play reverse past zero, abort

      case (0, _classPrivateFieldGet2.default)(this, _state).resumeSample <= 0 && nextDirection !== 1:
        return null;
      // attempting to play past the end, abort

      case (0, _classPrivateFieldGet2.default)(this, _state).resumeSample >= (0, _classPrivateFieldGet2.default)(this, _totalSamples) && nextDirection !== -1:
        return null;
      // valid nextState, return nextState params object

      default:
        return {
          nextDirection,
          nextPlaying: nextState === null || nextState === void 0 ? void 0 : nextState.playing,
          nextScrubbing: nextState === null || nextState === void 0 ? void 0 : nextState.scrubbing
        };
    }

    ;
  }

  /* assign nextState values to state */
  _transport_applyNextState() {
    var nextState = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
    (0, _classPrivateFieldGet2.default)(this, _state).direction = nextState.nextDirection;
    (0, _classPrivateFieldGet2.default)(this, _state).scrubbing = nextState.nextScrubbing;
    (0, _classPrivateFieldGet2.default)(this, _state).playing = nextState.nextPlaying;
  }

  /* clear pending queue and populate with ramp chunks transitioning to new state */
  _transport_scheduleDeltaChunks() {
    var _arguments4 = arguments,
        _this14 = this;

    return (0, _asyncToGenerator2.default)(function* () {
      var {
        nextDirection,
        nextPlaying,
        nextScrubbing
      } = _arguments4.length > 0 && _arguments4[0] !== undefined ? _arguments4[0] : {};

      _this14._queue_clearPending();

      var {
        startSpeed,
        startDirection,
        srcStartSample
      } = _this14._transport_getInitParams();

      var endSpeed = _this14._transport_calcEndSpeed(nextPlaying, nextScrubbing);

      if (nextDirection === startDirection) {
        // change speed in same direction
        yield _this14._transport_pushRamp({
          startSpeed: startSpeed,
          endSpeed: endSpeed,
          srcStartSample: srcStartSample,
          direction: nextDirection,
          ignoreGain: startSpeed !== (0, _classPrivateFieldGet2.default)(_this14, _playbackSpeeds).min && endSpeed !== (0, _classPrivateFieldGet2.default)(_this14, _playbackSpeeds).min
        });
      } else {
        // change directions
        var midSample = yield _this14._transport_pushRamp({
          startSpeed: startSpeed,
          endSpeed: (0, _classPrivateFieldGet2.default)(_this14, _playbackSpeeds).min,
          srcStartSample: srcStartSample,
          direction: startDirection
        });
        yield _this14._transport_pushRamp({
          startSpeed: (0, _classPrivateFieldGet2.default)(_this14, _playbackSpeeds).min,
          endSpeed: endSpeed,
          srcStartSample: midSample ? midSample : srcStartSample,
          direction: nextDirection
        });
      }

      ;
    })();
  }

  _transport_getInitParams() {
    var _lastScheduledChunk$c3, _lastScheduledChunk$d, _lastScheduledChunk$s, _lastScheduledChunk$c4;

    var lastScheduledChunk = this._lastScheduledChunk;
    return {
      startSpeed: (_lastScheduledChunk$c3 = lastScheduledChunk === null || lastScheduledChunk === void 0 ? void 0 : lastScheduledChunk.ctxPlaybackSpeed) !== null && _lastScheduledChunk$c3 !== void 0 ? _lastScheduledChunk$c3 : (0, _classPrivateFieldGet2.default)(this, _playbackSpeeds).min,
      startDirection: (_lastScheduledChunk$d = lastScheduledChunk === null || lastScheduledChunk === void 0 ? void 0 : lastScheduledChunk.direction) !== null && _lastScheduledChunk$d !== void 0 ? _lastScheduledChunk$d : (0, _classPrivateFieldGet2.default)(this, _state).direction,
      srcStartSample: (_lastScheduledChunk$s = lastScheduledChunk === null || lastScheduledChunk === void 0 ? void 0 : lastScheduledChunk.srcEndSample) !== null && _lastScheduledChunk$s !== void 0 ? _lastScheduledChunk$s : (0, _classPrivateFieldGet2.default)(this, _state).resumeSample,
      ctxStartGain: (_lastScheduledChunk$c4 = lastScheduledChunk === null || lastScheduledChunk === void 0 ? void 0 : lastScheduledChunk.ctxGain) !== null && _lastScheduledChunk$c4 !== void 0 ? _lastScheduledChunk$c4 : 0
    };
  }

  _transport_calcEndSpeed(nextPlaying, nextScrubbing) {
    var endSpeed = (0, _classPrivateFieldGet2.default)(this, _playbackSpeeds).min;

    if (nextPlaying === 1) {
      if (nextScrubbing === 1) {
        endSpeed = (0, _classPrivateFieldGet2.default)(this, _playbackSpeeds).scrub;
      }

      ;

      if (nextScrubbing === 0) {
        endSpeed = (0, _classPrivateFieldGet2.default)(this, _playbackSpeeds).base;
      }

      ;
    }

    ;
    return endSpeed;
  }

  _transport_pushRamp() {
    var _arguments5 = arguments,
        _this15 = this;

    return (0, _asyncToGenerator2.default)(function* () {
      var {
        startSpeed,
        endSpeed,
        srcStartSample,
        direction = 1,
        ignoreGain = false
      } = _arguments5.length > 0 && _arguments5[0] !== undefined ? _arguments5[0] : {};

      var rampChunkParams = _this15._transport_calcRampChunkParams(startSpeed, endSpeed);

      var nextStartSample = srcStartSample;

      while (rampChunkParams.length) {
        var startSample = nextStartSample;
        var {
          lengthSrcSamples,
          speed,
          gain
        } = rampChunkParams.shift();
        nextStartSample += lengthSrcSamples * direction;
        var chunk = yield _this15._chunk_create(startSample, nextStartSample, speed);
        if (!chunk) break;

        if (!ignoreGain) {
          chunk.ctxGain = gain;
        }

        ;
        (0, _classPrivateFieldGet2.default)(_this15, _pendingQueue).push(chunk);

        if (!rampChunkParams.length) {
          return chunk.srcEndSample;
        }

        ;
      }

      ;
    })();
  }

  _transport_calcRampChunkParams(startSpeed, endSpeed) {
    var speedStep = this._transport_calcRampSpeedStep(startSpeed, endSpeed);

    var rampChunkParams = [];
    var nextSpeed = startSpeed;

    while (true) {
      var speed = this._clampSpeed(nextSpeed);

      var gain = Math.max(0, Math.min(1, speed - (0, _classPrivateFieldGet2.default)(this, _playbackSpeeds).min));
      var lengthSrcSamples = (0, _classPrivateFieldGet2.default)(this, _rampChunkSamples) * speed;
      rampChunkParams.push({
        speed,
        lengthSrcSamples,
        gain
      });
      if (speed === endSpeed) break;

      if (Math.abs(endSpeed - nextSpeed) < Math.abs(speedStep)) {
        nextSpeed = endSpeed;
        continue;
      }

      ;
      nextSpeed += speedStep;
    }

    ;
    return rampChunkParams;
  }

  _transport_calcRampSpeedStep(startSpeed, endSpeed) {
    var lengthSeconds = this.constructor.calcRampDuration(startSpeed, endSpeed);
    var lengthSamples = lengthSeconds * (0, _classPrivateFieldGet2.default)(this, _sampleRate);
    var lengthRampChunks = Math.floor(lengthSamples / (0, _classPrivateFieldGet2.default)(this, _rampChunkSamples));
    var speedDelta = endSpeed - startSpeed;
    return speedDelta / lengthRampChunks;
  }

  /* ------------------------------------------------------------------ */

  /* Misc Helpers */

  /* clamp seconds value to whole-sample equivalent float */
  _clampClock(seconds) {
    var timeSamples = seconds * (0, _classPrivateFieldGet2.default)(this, _sampleRate);
    var timeSamplesWhole = Math.round(timeSamples);
    return timeSamplesWhole / (0, _classPrivateFieldGet2.default)(this, _sampleRate);
  }

  /* clamp speed value to whole-sample equivalent float */
  _clampSpeed(speed) {
    var safeSpeed = this.constructor.clampMinValidNumber(speed, this.constructor.MIN_PLAYBACK_SPEED);
    var srcChunkSamples = (0, _classPrivateFieldGet2.default)(this, _rampChunkSamples) * safeSpeed;
    var targetSrcChunkSamples = Math.round(srcChunkSamples);
    var targetSpeed = targetSrcChunkSamples / (0, _classPrivateFieldGet2.default)(this, _rampChunkSamples);
    return targetSpeed;
  }

  /* clamp arbitrary sample value to integer within available range */
  _clampSample(sample) {
    if (sample < 0) {
      return 0;
    }

    ;

    if (sample >= (0, _classPrivateFieldGet2.default)(this, _totalSamples)) {
      return (0, _classPrivateFieldGet2.default)(this, _totalSamples);
    }

    ;
    return parseInt(sample, 10);
  }

}

exports.default = Player;
(0, _defineProperty2.default)(Player, "DEFAULT_SAMPLE_RATE", 48e3);
(0, _defineProperty2.default)(Player, "DEFAULT_CHUNK_LENGTH", .02);
(0, _defineProperty2.default)(Player, "DEFAULT_LOOKAHEAD", 10);
(0, _defineProperty2.default)(Player, "DEFAULT_LATENCY", .1);
(0, _defineProperty2.default)(Player, "DEFAULT_PLAYBACK_SPEED", 1);
(0, _defineProperty2.default)(Player, "DEFAULT_SCRUB_SPEED", 8);
(0, _defineProperty2.default)(Player, "MIN_LOOKAHEAD_SECONDS", 1);
(0, _defineProperty2.default)(Player, "MIN_SCHEDULED_SECONDS", .1);
(0, _defineProperty2.default)(Player, "MIN_PLAYBACK_SPEED", .1);
(0, _defineProperty2.default)(Player, "KBIN_SAMPLES", 128);
(0, _defineProperty2.default)(Player, "MIN_RAMP_CHUNK_KBINS", 4);
(0, _defineProperty2.default)(Player, "TRANSPORT", {
  play: {
    playing: 1,
    scrubbing: 0,
    direction: 1
  },
  stop: {
    playing: 0,
    scrubbing: 0,
    direction: 1
  },
  rev: {
    playing: 1,
    scrubbing: 0,
    direction: -1
  },
  ff: {
    playing: 1,
    scrubbing: 1,
    direction: 1
  },
  rew: {
    playing: 1,
    scrubbing: 1,
    direction: -1
  }
});
;